<?php
/**
* 出入库单据相关处理
*/
class console_receipt_vopstock
{
    private $_stockoutObj = null;

    private $_stockItemObj = null;

    private $_isoInfo = array();

    private $_items = array();
    
    private static $iso_status =array(
        'PARTIN'=>2 ,
        'FINISH'=>3,
        'CLOSE'=>4,
        'CANCEL'=>4,
        'FAILED'=>4,
    );

    function __construct()
    {
        //$this->_basicMStockLib    = kernel::single('material_basic_material_stock');
        //$this->_branproductObj    = kernel::single('ome_branch_product');
        
        $this->_stockoutObj    = app::get('purchase')->model('pick_stockout_bills');
        $this->_stockItemObj   = app::get('purchase')->model('pick_stockout_bill_items');
        $this->_pickObj        = app::get('purchase')->model('pick_bills');
    }

    /**
     * 检查出入库单是否有效
     * 
     * @param  $iso_bn 出入库单编号
     * @param $status 需要执行状态
     * 
     * @msg 返回结果
     */
    public function checkValid($iso_bn, $status, &$msg)
    {
        $iso    = $this->checkExist($iso_bn);
        
        if (!$iso){
            $msg = '单据号不存在!';
            return false;
        }
        
        $iso_status   = $iso['status'];//单据状态
        $o_status     = $iso['o_status'];//出库状态
        
        switch($status){
            case 'PARTIN':
            case 'FINISH':
                if($iso_status == 3)
                {
                    $msg = '单据已完成,不可以入库';
                    return false;
                }
                elseif($iso_status == 2)
                {
                    $msg = '单据已取消，不可以入库';
                    return false;
                }
                elseif($o_status == 3)
                {
                    $msg = '已全部出库,不可以入库';
                    return false;
                }
                break;
            case 'CANCEL':
            case 'CLOSE':
                if ($o_status=='3' || $o_status=='2')
                {
                    $msg = '单据已部分或全部入库,不可以取消';
                    return false;
                }elseif ($iso_status==2){
                    $msg = '单据已取消，不可以再次取消';
                    return false;
                }
                break;
        }
        
        return true;
    }
    
    /**
     * 判断出入库明细是否存在
     *
     * @param string $stockout_no
     * @return Array
     */
    public function checkExist($stockout_no)
    {
        $iso    = $this->_stockoutObj->dump(array('stockout_no'=>$stockout_no), '*');
        if($iso)
        {
            //出库类型
            $iostock_instance  = kernel::single('siso_receipt_iostock');
            $iso['type_id']    = $iostock_instance::VOP_STOCKOUT;
            
            $this->_isoInfo    = $iso;
            
            //出库单明细
            $sql        = "SELECT a.*, b.bm_id AS product_id FROM sdb_purchase_pick_stockout_bill_items AS a 
                           LEFT JOIN sdb_material_basic_material AS b ON a.bn=b.material_bn WHERE a.stockout_id=". $iso['stockout_id'];
            $_items     = $this->_stockoutObj->db->select($sql);
            
            $this->_items    = array();
            foreach ($_items as $k=>$item)
            {
                $this->_items[$item['bill_id']][$item['bn']] = $item;
            }
        }
        
        return $iso;
    }
    
    /**
     * 出入库据保存(io为0出库,1入库)
     * 
     * @param Array $data
     * @param intval $io
     * @param string $msg
     * 
     * @return boolean
     */
    public function do_save($data, $io, &$msg)
    {
        set_time_limit(0);
        
        $iostock_update    = true;
        $auto_iostock      = false;
        $io_status         = $data['io_status'];
        
        //出库货品操作
        $iostockData    = $this->_format_items($data, $io, $iostock_update, $auto_iostock);
        
        //出库数量大于可实际数量
        if($auto_iostock == true)
        {
            $msg           = '因仓库反馈数量大于申请数量,人工介入';
            $save_data     = array('receive_status'=>console_const::_FINISH_CODE);
            $iso_result    = $this->_stockoutObj->update($save_data, array('stockout_id'=>$this->_isoInfo['stockout_id']));
            
            return true;
        }
        
        $stockLib    = kernel::single('siso_receipt_iostock_vopstockout');
        
        $stockLib->_typeId = $this->_isoInfo['type_id'];
        
        //开启事务
        $this->_stockoutObj->db->exec('begin');
        
        //生成出入库明细
        if($iostockData['items'])
        {
            $result = $stockLib->create($iostockData, $data, $msg);
            
            if (!$result){
                $this->_stockoutObj->db->exec('rollback');
                $msg = '出入库失败';
                return false;
            }
        }
        
        //更新出库单
        $save_flag    = $this->update_stockout($io_status, $msg);
        if(!$save_flag)
        {
            $this->_stockoutObj->db->exec('rollback');
            return false;
        }
        
        //提交事务
        $this->_stockoutObj->db->exec('commit');
        
        //释放冻结
        if($io == '0')
        {
            $this->clear_stockout_store_freeze($iostockData, $io_status);
        }
        
        return true;
    }
    
    /**
     * 格式化数据
     * 
     * $iostock_update 是否需要确认
     * $auto_iostock 出入库数量大于申请数量时不自动完成
     */
    function _format_items($data, $io, &$iostock_update, &$auto_iostock)
    {
        $iso_item       = array();
        $stockout_id    = $this->_isoInfo['stockout_id'];
        
        //更新出库货品数量
        $temp_bn_num     = array();
        $temp_actual_num = array();
        $bill_true_num   = array();
        
        foreach ($data['items'] as $key => $item)
        {
            $pick_bn  = $item['pick_bn'];
            $bn       = $item['bn'];
            $num      = $item['num'];
            
            //拣货单信息
            $pickInfo    = $this->_pickObj->dump(array('pick_no'=>$pick_bn), 'bill_id, po_id');
            if(empty($pickInfo))
            {
                return false;
            }
            $po_id    = $pickInfo['po_id'];
            $bill_id  = $pickInfo['bill_id'];
            
            //出库货品信息
            if($this->_items[$bill_id][$bn])
            {
                $true_num    = $this->_items[$bill_id][$bn]['num'] - $this->_items[$bill_id][$bn]['actual_num'];
                if($num > $true_num)
                {
                    $auto_iostock    = true;//因仓库反馈数量大于申请数量,人工介入
                }
                
                $stockout_item_id  = $this->_items[$bill_id][$bn]['stockout_item_id'];
                $price             = $this->_items[$bill_id][$bn]['price'];
                $product_id        = $this->_items[$bill_id][$bn]['product_id'];
                $price             = $this->_items[$bill_id][$bn]['price'];
                
                //更新
                $sql    = "UPDATE sdb_purchase_pick_stockout_bill_items SET actual_num=actual_num+". $num ." 
                           WHERE stockout_item_id=". $stockout_item_id ." AND stockout_id=". $stockout_id;
                $this->_stockItemObj->db->exec($sql);
                
                //请求出库数量
                $temp_bn_num[$bn]  += $num;
                
                //已出库数量(算上本次出库数量)
                if($temp_actual_num[$bn])
                {
                    $temp_actual_num[$bn] += $num;
                }
                else
                {
                    $temp_actual_num[$bn] = $this->_items[$bill_id][$bn]['actual_num'] + $num;
                }
                
                //可出库数量
                $bill_true_num[$bn][$bill_id]    = $true_num;
                
                //组织数据
                $iso_item[$bn]    = array(
                                'bn'=>$bn,
                                'price'=>$price,
                                'iso_items_id'=>$stockout_item_id,
                                'product_id'=>$product_id,
                                'nums'=>$temp_bn_num[$bn],
                                'actual_num'=>$temp_actual_num[$bn],
                );
            }
        }
        
        //[格式化]可出库数量
        foreach ($bill_true_num as $bn => $val)
        {
            $temp_true_num    = 0;
            
            foreach ($val as $true_num)
            {
                $temp_true_num    += $true_num;
            }
            
            $iso_item[$bn]['effective_num']    = $temp_true_num;
        }
        
        //出库单信息
        $iostock  = array(
                'iso_id'=>$this->_isoInfo['stockout_id'],
                'stockout_id'=>$this->_isoInfo['stockout_id'],
                'type_id'=>$this->_isoInfo['type_id'],
                'iso_bn'=>$this->_isoInfo['stockout_no'],
                'branch_id'=>$this->_isoInfo['branch_id'],
                'memo'=>$data['memo'],
                'operate_time'=>$data['operate_time'],
        );
        $iostock['items']    = $iso_item;
        
        return $iostock;
    }
    
    /**
     * 释放冻结库存
     *
     * array data 当有明细时,操作对应明细，否则操作所有
     */
    public function clear_stockout_store_freeze($data, $io_status='')
    {
        $branch_id = $data['branch_id'];
        $items     = $data['items'];
        
        //库存管控处理
        $storeManageLib    = kernel::single('ome_store_manage');
        $storeManageLib->loadBranch(array('branch_id'=>$branch_id));
        
        $params    = array();
        $params['node_type'] = 'finishVopstockout';
        $params['params']    = array('stockout_id'=>$data['stockout_id'], 'branch_id'=>$branch_id);
        
        //装箱出库明细
        foreach((array)$items as $item)
        {
            if($item['nums'] <= 0) continue;
            
            $num              = $item['nums'];#请求出库数量
            $effective_num    = $item['effective_num'];#可出库数量
            $product_id       = $item['product_id'];
            
            //释放冻结库存
            if($num > 0)
            {
                //库存管控处理
                $params['params']['product_id'] = $product_id;
                $params['params']['num']        = $num;
                
                $processResult    = $storeManageLib->processBranchStore($params, $err_msg);
            }
        }
        
        //将未产生过出入库记录释放冻结
        if($io_status == 'FINISH')
        {
            //获取最新的出库单明细
            $_items     = array();
            $sql        = "SELECT a.*, b.bm_id FROM sdb_purchase_pick_stockout_bill_items AS a 
                           LEFT JOIN sdb_material_basic_material AS b ON a.bn=b.material_bn WHERE a.stockout_id=". $this->_isoInfo['stockout_id'];
            $_items     = $this->_stockoutObj->db->select($sql);
            
            //获取出入库单上的货号
            $iostock_list    = $this->getIostockList($this->_isoInfo['type_id'], $data['stockout_id']);
            
            foreach ($_items as $_item)
            {
                $num = 0;
                
                //部分出库后释放未出库的数量
                if($_item['actual_num']==0 && !in_array($_item['bn'],$iostock_list))
                {
                    $num    = $_item['num'] - $_item['actual_num'];//未出库的数量
                }
                elseif(($_item['actual_num']>0) && ($_item['num']>$_item['actual_num']))
                {
                    $num    = $_item['num'] - $_item['actual_num'];//剩余未出库的数量
                }
                
                //释放冻结库存
                if ($num>0)
                {
                    //库存管控处理
                    $params['params']['product_id'] = $_item['bm_id'];
                    $params['params']['num']        = $num;
                    
                    $processResult    = $storeManageLib->processBranchStore($params, $err_msg);
                }
            }
        }
        
        //删除预占流水
        if($io_status=='FINISH' || $io_status=='')
        {
            $basicMStockFreezeLib = kernel::single('material_basic_material_stock_freeze');
            $basicMStockFreezeLib->delOtherFreeze($data['stockout_id'], material_basic_material_stock_freeze::__VOPSTOCKOUT);
        }
    }
    
    /**
     * 获取出入库单上的货号
     */
    function getIostockList($type_id, $stockout_id)
    {
        $items = array();
        
        $sql = "SELECT bn FROM sdb_ome_iostock WHERE type_id in(". $type_id .") AND original_id=". $stockout_id ." AND nums>0";
        $iso = $this->_stockoutObj->db->select($sql);
        
        foreach ($iso as $v)
        {
            $items[]=$v['bn'];
        }
        
        return $items;
    }
    
    /**
     *
     * 出入库单取消
     * @param $io_bn 出入库单号
     * @param $io 出 入标识
     */
    public function cancel($stockout_no, $io)
    {
        return false;//暂时没有取消操作
    }
    
    /**
     * 更新出库单的出库状态为部分出库
     *
     * @param Array $data
     * @param intval $io
     * @param string $msg
     *
     * @return boolean
     */
    public function save_stockout($data, $io, &$msg)
    {
        $result    = $this->_stockoutObj->update(array('o_status'=>2), array('stockout_id'=>$this->_isoInfo['stockout_id']));
        
        return $result;
    }
    
    public function update_stockout($io_status, &$msg)
    {
        $logObj      = app::get('ome')->model('operation_log');
        
        //获取出库状态
        $o_status   = self::$iso_status[$io_status];
        $status     = 3;//单据状态完成
        
        //单据状态
        if($io_status == 'PARTIN')
        {
            $status     = 1;//未完成
        }
        
        //确认出库状态
        if($io_status == 'FINISH')
        {
            $sql     = "SELECT stockout_item_id FROM sdb_purchase_pick_stockout_bill_items
                        WHERE stockout_id=". $this->_isoInfo['stockout_id'] ." AND actual_num<num";
            $itemRow = $this->_stockoutObj->db->selectRow($sql);
            if($itemRow)
            {
                $io_status   = 'PARTIN';
                $o_status    = self::$iso_status[$io_status];//部分出库
            }
        }
        
        //[更新]出库单
        $save_data    = array('o_status'=>$o_status, 'status'=>$status, 'last_modified'=>time());
        $iso_result   = $this->_stockoutObj->update($save_data, array('stockout_id'=>$this->_isoInfo['stockout_id']));
        if(!$iso_result)
        {
            //logs
            $msg    = '出库单据状态更新失败';
            $logObj->write_log('check_stockout_bills@ome', $this->_isoInfo['stockout_id'], $msg);
            
            return false;
        }
        
        //[更新]拣货单和采购单(注意：多个拣货单可合并成一个出库单)
        $sql     = "SELECT a.bill_id, a.po_id, b.stockout_id FROM sdb_purchase_pick_bills AS a
                    LEFT JOIN sdb_purchase_pick_stockout AS b ON a.bill_id=b.bill_id
                    WHERE b.stockout_id=". $this->_isoInfo['stockout_id'];
        $pickList    = $this->_stockoutObj->db->select($sql);
        foreach ($pickList as $key => $val)
        {
            $bill_id     = $val['bill_id'];
            $po_id       = $val['po_id'];
            $stockout_id = $val['stockout_id'];
            
            $item_sql    = "SELECT sum(actual_num) AS num FROM sdb_purchase_pick_stockout_bill_items 
                            WHERE stockout_id=". $stockout_id ." AND po_id=". $po_id ." AND bill_id=". $bill_id;
            $branch_send_num    = $this->_stockoutObj->db->selectrow($item_sql);
            $branch_send_num    = $branch_send_num['num'];
            
            //拣货单更新
            $up_sql    = "UPDATE sdb_purchase_pick_bills SET delivery_status=2, branch_send_num=branch_send_num + ". $branch_send_num .", last_modified=". time() ." 
                          WHERE bill_id=". $bill_id;
            $result    = $this->_pickObj->db->exec($up_sql);
            if(!$result)
            {
                //logs
                $msg    = '拣货单更新失败';
                $logObj->write_log('check_stockout_bills@ome', $this->_isoInfo['stockout_id'], $msg);
                
                return false;
            }
            
            //logs
            $logObj->write_log('check_vopick@ome', $bill_id, '拣货单发货完成');
        }
        
        //logs
        $logObj->write_log('check_stockout_bills@ome', $this->_isoInfo['stockout_id'], '出库单处理成功');
        
        return true;
    }
}
?>