<?php
/**
 * 唯品会JIT相关接口
 */
class erpapi_shop_request_purchase extends erpapi_shop_request_abstract {

    /**
     * 组织接口参数
     * 
     * @param string  $vop_method  接口名
     * @param intval  $is_multiple_params 是否多个参数,默认1
     * @param Array   data 查询条件
     * @return Array
     */
    public function _format_api_params($vop_method, $is_multiple_params=1, $data)
    {
        $vop_param                        = array();
        $vop_param['vop_method']          = $vop_method;
        $vop_param['is_multiple_params']  = $is_multiple_params;
        $vop_param['data']                = ($data ? json_encode($data) : '{}');
        
        return $vop_param;
    }
    
    /**
     * 手工单拉PO单号
     */
    public function getPullPo($param)
    {
        if(empty($param['po_no']))
        {
            return false;
        }
        
        //组织参数getPoList
        $vop_param    = $this->_format_api_params('getPoList', 1, $param);
        
        $title      = '获取采购单号('. $param['po_no'] .')信息';
        $primary_bn = $param['po_no'];
        
        $rsp    = $this->__caller->call(SHOP_COMMONS_VOP_JIT, $vop_param, array(), $title, 10, $primary_bn);
        
        //数据处理
        if($rsp['rsp'] == 'succ')
        {
            $rsp    = $this->getPoList_callback($rsp);
        }
        
        return $rsp;
    }
    
    /**
     * 获取po列表
     */
    public function getPoList($param)
    {
        //组织参数
        $vop_param    = $this->_format_api_params('getPoList', 1, $param);
        
        $title        = '获取店铺('. $this->__channelObj->channel['name'] .')PO单列表';
        $primary_bn   = 'getPoList';
        
        $shop_info    = array('shop_id'=>$this->__channelObj->channel['shop_id']);
        
        $rsp    = $this->__caller->call(SHOP_COMMONS_VOP_JIT, $vop_param, array(), $title, 10, $primary_bn);
        
        //数据处理
        if($rsp['rsp'] == 'succ')
        {
            $rsp    = $this->getPoList_callback($rsp);
        }
        
        return $rsp;
    }
    
    /**
     * 回调
     */
    public function getPoList_callback($response, $callback_params=NULL)
    {
        $rsp        = $response['rsp'];
        $shop_id    = $this->__channelObj->channel['shop_id'];
        $is_empty   = false;
        
        if($rsp == 'succ')
        {
            $purchaseObj    = app::get('purchase')->model('order');
            $purchaseLib    = kernel::single('purchase_purchase_order');
            
            $data    = json_decode($response['data'], true);
            $data    = json_decode($data['msg'], true);
            
            $po_list = $data['result']['purchase_order_list'];//接口名getPoList
            
            $pick_pos    = array();//需要获取拣货单的PO单号
            
            //是否为空
            $is_empty    = (empty($po_list) ? true : false);
            
            if($po_list && is_array($po_list))
            {
                foreach ($po_list as $key => $val)
                {
                    $val['sell_st_time']    = ($val['sell_st_time'] ? strtotime($val['sell_st_time']) : 0);
                    $val['sell_et_time']    = ($val['sell_et_time'] ? strtotime($val['sell_et_time']) : 0);
                    
                    $poInfo    = $purchaseObj->dump(array('po_bn'=>$val['po_no']), 'po_id, sales_num, unpick_num');
                    if($poInfo)
                    {
                        //更新
                        $data    = array(
                                'po_id'=>$poInfo['po_id'],
                                'stock'=> intval($val['stock']),
                                'sales_num'=> intval($val['sales_volume']),
                                'unpick_num'=> intval($val['not_pick']),
                                'old_sales_num'=>$poInfo['sales_num'],
                                'old_unpick_num'=>$poInfo['unpick_num'],
                        );
                        $result    = $purchaseLib->update_purchase($data);
                    }
                    else 
                    {
                        //新建
                        $data    = array(
                                'po_bn'=>$val['po_no'],//采购单号
                                'shop_id'=> $shop_id,//来源店铺
                                'co_mode'=> $val['co_mode'],
                                'sell_st_time'=>$val['sell_st_time'],//档期开始时间
                                'sell_et_time'=>$val['sell_et_time'],//档期结束时间
                                'stock'=> intval($val['stock']),//虚拟总库存
                                'sales_num'=> intval($val['sales_volume']),//销售数量
                                'unpick_num'=> intval($val['not_pick']),//未拣货数量
                                'trade_mode'=>$val['trade_mode'],
                                'schedule_id'=>$val['schedule_id'],//档期号
                                'schedule_name'=>$val['schedule_name'],//档期名称
                                'supplier_name'=>$val['vendor_name'],//供应商名称
                                'brand_name'=>$val['brand_name'],//品牌名称
                                'warehouse'=>$val['warehouse'],//仓库
                                'is_normal'=>intval($val['normality_flag']),//是否是常态档期
                        );
                        $result    = $purchaseLib->create_purchase($data);
                    }
                    
                    //[有未拣货数量]通过Po单号获取拣货单列表
                    if($result && $data['unpick_num'])
                    {
                        $pick_pos[]    = $val['po_no'];
                    }
                }
            }
        }
        
        $result    = array('rsp'=>$response['rsp'], 'msg_id'=>$response['msg_id'], 'res'=>$response['res'], 'is_empty'=>$is_empty);
        if($pick_pos)
        {
            $result['po_nos']    = $pick_pos;
        }
        
        return $result;
    }
    
    /**
     * 获取拣货单列表
     */
    public function getPickList($param)
    {
        if(empty($param))
        {
            return array('rsp'=>'fail', 'err_msg'=>'没有输入参数');
        }
        
        //组织参数
        $vop_param    = $this->_format_api_params('getPickList', 1, $param);
        
        if($param['pick_no'])
        {
            $title  = '获取拣货单号('. $param['pick_no'] .')信息';
            $primary_bn = $param['pick_no'];
        }
        else
        {
            $title        = '获取店铺('. $this->__channelObj->channel['name'] .')拣货单列表';
            $primary_bn   = 'getPickList';
        }
        
        $rsp    = $this->__caller->call(SHOP_COMMONS_VOP_JIT, $vop_param, array(), $title, 10, $primary_bn);
        
        //数据处理
        if($rsp['rsp'] == 'succ')
        {
            $rsp    = $this->getPickList_callback($rsp);
        }
        
        return $rsp;
    }
    
    /**
     * 拣货单回调
     */
    public function getPickList_callback($response, $callback_params=NULL)
    {
        $rsp    = $response['rsp'];
        
        if($rsp == 'succ')
        {
            $data    = json_decode($response['data'], true);
            $data    = json_decode($data['msg'], true);
            $pick_list = $data['result']['picks'];
            
            $pick_nos    = array();
            
            //获取拣货单明细
            if($pick_list)
            {
                foreach ($pick_list as $key => $val)
                {
                    //组织拣货单参数
                    $pick_nos[$val['pick_no']]    = $val;
                }
            }
        }
        
        $result    = array('rsp'=>$response['rsp'], 'msg_id'=>$response['msg_id'], 'res'=>$response['res']);
        if($pick_nos)
        {
            $result['po_list']    = $pick_nos;
        }
        
        return $result;
    }
    
    /**
     * 拣货单明细
     */
    public function getPickDetail($param)
    {
        if(empty($param))
        {
            return array('rsp'=>'fail', 'err_msg'=>'没有输入参数');
        }
        
        //组织参数
        $vop_param    = $this->_format_api_params('getPickDetail', 1, $param);
        
        $title      = '获取拣货单号('. $param['pick_no'] .')明细信息';
        $primary_bn = $param['pick_no'];
        
        $rsp    = $this->__caller->call(SHOP_COMMONS_VOP_JIT, $vop_param, array(), $title, 10, $primary_bn);
        
        //数据处理
        if($rsp['rsp'] == 'succ')
        {
            $rsp    = $this->getPickDetail_callback($rsp);
        }
        
        return $rsp;
    }
    
    /**
     * 拣货单明细回调
     */
    public function getPickDetail_callback($response, $callback_params=NULL)
    {
        $rsp          = $response['rsp'];
        $pick_data    = array();
        
        if($rsp == 'succ')
        {
            $data    = json_decode($response['data'], true);
            $data    = json_decode($data['msg'], true);
            
            $pick_product_list    = $data['result']['pick_product_list'];
        }
        
        $result    = array('rsp'=>$response['rsp'], 'msg_id'=>$response['msg_id'], 'res'=>$response['res']);
        if($pick_product_list)
        {
            $result['pick_product_list']    = $pick_product_list;
        }
        
        return $result;
    }
    
    /**
     * 创建拣货单
     */
    public function createPick($param)
    {
        if(empty($param['po_no']))
        {
            return false;
        }
        
        //组织参数
        $vop_param    = $this->_format_api_params('createPick', 1, $param);
        
        $title      = '采购单号('. $param['po_no'] .')创建拣货单';
        $primary_bn = $param['po_no'];
        
        $rsp    = $this->__caller->call(SHOP_COMMONS_VOP_JIT, $vop_param, array(), $title, 10, $primary_bn);
        
        //数据处理
        if($rsp['rsp'] == 'succ')
        {
            $rsp    = $this->createPick_callback($rsp);
        }
        
        return $rsp;
    }
    
    /**
     * 创建拣货单回调
     */
    public function createPick_callback($response, $callback_params=NULL)
    {
        $rsp          = $response['rsp'];
        $pick_info    = array();
        
        if($rsp == 'succ')
        {
            $data    = json_decode($response['data'], true);
            $data    = json_decode($data['msg'], true);
            
            $pick_info    = $data['result'];
        }
        
        $result    = array('rsp'=>$response['rsp'], 'msg_id'=>$response['msg_id'], 'res'=>$response['res']);
        if($pick_info)
        {
            $result['pick_info']    = $pick_info;
        }
        
        return $result;
    }
    
    /**
     * 创建出仓单
     */
    public function createDelivery($param)
    {
        if(empty($param['po_no']) || empty($param['delivery_no']) || empty($param['warehouse']))
        {
            return false;
        }
        if(empty($param['arrival_time']) || empty($param['carrier_name']) || empty($param['carrier_code']))
        {
            return false;
        }
        
        //组织参数
        $vop_param    = $this->_format_api_params('createDelivery', 1, $param);
        
        $title        = '出库单号('. $param['delivery_no'] .')创建出库单';
        $primary_bn   = $param['delivery_no'];
        
        $rsp    = $this->__caller->call(SHOP_COMMONS_VOP_JIT, $vop_param, array(), $title, 10, $primary_bn);
        
        //数据处理
        if($rsp['rsp'] == 'succ')
        {
            $rsp    = $this->createDelivery_callback($rsp);
        }
        
        return $rsp;
    }
    
    /**
     * 创建出仓单回调
     */
    public function createDelivery_callback($response, $callback_params=NULL)
    {
        $rsp          = $response['rsp'];
        $delivery     = array();
        
        if($rsp == 'succ')
        {
            $data    = json_decode($response['data'], true);
            $data    = json_decode($data['msg'], true);
            $data    = $data['result'];
            
            $delivery    = array('delivery_id'=>$data['delivery_id'], 'storage_no'=>$data['storage_no']);
        }
        
        $result    = array('rsp'=>$response['rsp'], 'msg_id'=>$response['msg_id'], 'res'=>$response['res']);
        if($delivery)
        {
            $result['delivery']    = $delivery;
        }
        
        return $result;
    }
    
    /**
     * 创建出仓单2.0
     */
    public function createMultiPoDelivery($param)
    {
        $chk_param    = $param['createMultiPoDeliveryRequest'];
        
        if(empty($chk_param['po_no']) || empty($chk_param['delivery_no']) || empty($chk_param['warehouse']))
        {
            return false;
        }
        if(empty($chk_param['arrival_time']) || empty($chk_param['carrier_code']))
        {
            return false;
        }
        
        //组织参数
        $vop_param    = $this->_format_api_params('createMultiPoDelivery', 0, $param);
        
        $title        = '出库单号('. $chk_param['delivery_no'] .')创建出库单';
        $primary_bn   = $chk_param['delivery_no'];
        
        $rsp    = $this->__caller->call(SHOP_COMMONS_VOP_JIT, $vop_param, array(), $title, 10, $primary_bn);
        
        //数据处理
        if($rsp['rsp'] == 'succ')
        {
            $rsp    = $this->createMultiPoDelivery_callback($rsp);
        }
        
        return $rsp;
    }
    
    /**
     * 创建出仓单回调2.0
     */
    public function createMultiPoDelivery_callback($response, $callback_params=NULL)
    {
        $rsp          = $response['rsp'];
        $delivery     = array();
        
        if($rsp == 'succ')
        {
            $data          = json_decode($response['data'], true);
            $data          = json_decode($data['msg'], true);
            $storage_no    = $data['result'];
            
            if($storage_no)
            {
                $delivery    = array('storage_no'=>$storage_no);
            }
        }
        
        $result    = array('rsp'=>$response['rsp'], 'msg_id'=>$response['msg_id'], 'res'=>$response['res']);
        if($delivery)
        {
            $result['delivery']    = $delivery;
        }
        
        return $result;
    }
    
    /**
     * 修改出仓单信息
     */
    public function editDelivery($param)
    {
        $stockout_no    = $param['stockout_no'];
        unset($param['stockout_no']);
        
        if(empty($stockout_no) || empty($param['storage_no']) || empty($param['warehouse']))
        {
            return false;
        }
        
        //组织参数
        $vop_param    = $this->_format_api_params('editDelivery', 1, $param);
        
        $title        = '出库单号('. $stockout_no .')修改出仓单信息';
        $primary_bn   = $stockout_no;
        
        $rsp    = $this->__caller->call(SHOP_COMMONS_VOP_JIT, $vop_param, array(), $title, 10, $primary_bn);
        
        //数据处理
        if($rsp['rsp'] == 'succ')
        {
            $rsp    = $this->editDelivery_callback($rsp);
        }
        
        return $rsp;
    }
    
    /**
     * 修改出仓单回调
     */
    public function editDelivery_callback($response, $callback_params=NULL)
    {
        $rsp          = $response['rsp'];
        $delivery     = array();
        
        if($rsp == 'succ')
        {
            $data    = json_decode($response['data'], true);
            $data    = json_decode($data['msg'], true);
            
            //失败原因
            if($data['returnCode'] != '0' && $data['returnMessage'])
            {
                $response['rsp']        = 'fail';
                $response['err_msg']    = json_encode(array('returnMessage'=>$data['returnMessage']));
            }
        }
        
        $result    = array('rsp'=>$response['rsp'], 'msg_id'=>$response['msg_id'], 'res'=>$response['res'], 'err_msg'=>$response['err_msg']);
        
        return $result;
    }
    
    /**
     * 将出仓明细信息导入到出仓单中（目前该接口明细信息最大导入量在500条SKU信息）
     */
    public function importDeliveryDetail($param)
    {
        $stockout_no    = $param['stockout_no'];
        unset($param['stockout_no']);
        
        if(empty($stockout_no) || empty($param['po_no']) || empty($param['storage_no']))
        {
            return false;
        }
        
        //组织参数
        $vop_param    = $this->_format_api_params('importDeliveryDetail', 1, $param);
        
        $title        = '出库单号('. $stockout_no .')导入出仓明细信息';
        $primary_bn   = $stockout_no;
        
        $rsp    = $this->__caller->call(SHOP_COMMONS_VOP_JIT, $vop_param, array(), $title, 10, $primary_bn);
        
        //数据处理
        if($rsp['rsp'] == 'succ')
        {
            $rsp    = $this->importDeliveryDetail_callback($rsp);
        }
        
        return $rsp;
    }
    
    /**
     * 导入出仓明细信息回调
     */
    public function importDeliveryDetail_callback($response, $callback_params=NULL)
    {
        $rsp          = $response['rsp'];
        $delivery     = array();
        
        if($rsp == 'succ')
        {
            $data    = json_decode($response['data'], true);
            $data    = json_decode($data['msg'], true);
            
            //失败原因
            if($data['returnCode'] != '0' && $data['returnMessage'])
            {
                $response['rsp']        = 'fail';
                $response['err_msg']    = json_encode(array('returnMessage'=>$data['returnMessage']));
            }
        }
        
        $result    = array('rsp'=>$response['rsp'], 'msg_id'=>$response['msg_id'], 'res'=>$response['res'], 'err_msg'=>$response['err_msg']);
        
        return $result;
    }
    
    /**
     * 确认出仓单
     */
    public function confirmDelivery($param)
    {
        $stockout_no    = $param['stockout_no'];
        unset($param['stockout_no']);
        
        if(empty($stockout_no) || empty($param['storage_no']))
        {
            return false;
        }
        
        //组织参数
        $vop_param    = $this->_format_api_params('confirmDelivery', 1, $param);
        
        $title        = '出库单号('. $stockout_no .')确认出仓单';
        $primary_bn   = $stockout_no;
        
        $rsp    = $this->__caller->call(SHOP_COMMONS_VOP_JIT, $vop_param, array(), $title, 10, $primary_bn);
        
        //数据处理
        if($rsp['rsp'] == 'succ')
        {
            $rsp    = $this->confirmDelivery_callback($rsp);
        }
        
        return $rsp;
    }
    
    /**
     * 确认出仓单回调
     */
    public function confirmDelivery_callback($response, $callback_params=NULL)
    {
        $rsp          = $response['rsp'];
        $delivery     = array();
        
        if($rsp == 'succ')
        {
            $data    = json_decode($response['data'], true);
            $data    = json_decode($data['msg'], true);
            
            //失败原因
            if($data['returnCode'] != '0' && $data['returnMessage'])
            {
                $response['rsp']        = 'fail';
                $response['err_msg']    = json_encode(array('returnMessage'=>$data['returnMessage']));
            }
        }
        
        $result    = array('rsp'=>$response['rsp'], 'msg_id'=>$response['msg_id'], 'res'=>$response['res'], 'err_msg'=>$response['err_msg']);
        
        return $result;
    }
    
    /**
     * 同步承运商
     */
    function getCarrierList($param)
    {
        $chk_param    = $param['carrierRequest'];
        $shop_id      = $this->__channelObj->channel['shop_id'];
        
        //组织参数
        $vop_param    = $this->_format_api_params('getCarrierList', 0, $param);
        
        $title        = '同步承运商';
        $primary_bn   = 'carrier';
        
        $rsp    = $this->__caller->call(SHOP_COMMONS_VOP_JIT, $vop_param, array(), $title, 10, $primary_bn);
        
        //数据处理
        if($rsp['rsp'] == 'succ')
        {
            $rsp    = $this->getCarrierList_callback($rsp, $shop_id);
        }
        
        return $rsp;
    }
    
    /**
     * 同步承运商回调
     */
    public function getCarrierList_callback($response, $shop_id)
    {
        $rsp          = $response['rsp'];
        $delivery     = array();
        
        if($rsp == 'succ')
        {
            $data          = json_decode($response['data'], true);
            $data          = json_decode($data['msg'], true);
            $dataList      = $data['result']['carriers'];
            
            if($dataList)
            {
                $stockLib        = kernel::single('purchase_purchase_stockout');
                
                foreach ($dataList as $key => $val)
                {
                    $save_data    = array(
                            'tms_carrier_id'=>$val['tms_carrier_id'],
                            'carrier_code'=>$val['carrier_code'],
                            'carrier_name'=>$val['carrier_name'],
                            'carrier_shortname'=>$val['carrier_shortname'],
                            'carrier_isvalid'=>$val['carrier_isvalid'],
                            'shop_id'=>$shop_id,
                    );
                    
                    $stockLib->saveCarrier($save_data);
                }
            }
        }
        
        $result    = array('rsp'=>$response['rsp'], 'msg_id'=>$response['msg_id'], 'res'=>$response['res']);
        
        return $result;
    }
}